-- ПАРК-ЛАБИРИНТ
-- вход и выход: комната park_start
-- важно: на каждом ходе меняется картинка

-- ПАРК
park_start = room {
	nam = 'Начало парка';
	pic = function(s)
		return 'gfx/park/fog_lr.jpg'
	end;
	dsc = [[
		Мощёная дорога ведёт вглубь парка. Впереди туман. 
	]];
	way = { 'front_side', 'park' };
};

--[[
	Сделаем мини-лабиринт размером [-2..3, 0..3]
	- нач.клетка (0, 0)
	- красный сундук (-1, 1)
	- зеленый сундук (2, 1)	
	- синий сундук (1, 2)
	Все переходы разрешены, кроме выхода за границы участка
	Все переходы на тайники, кроме указанных на карте, 
	ведут на поле X (0, 2)

	Можно усложнить немного: 
	- синий сундук (0, 2)
	- точка X (-1, 2)
	но потребуется на рисунке дорисовать стрелку из (1,2) в (0,2)
]]

-- константы размеров лабиринта
local MIN_X = -2
local MAX_X = 3
local MIN_Y = 0
local MAX_Y = 3
-- координаты сундуков и направления, откуда должны прийти
local RED_X = -1
local RED_Y = 1
local RED_DIR = 1	-- up
local GREEN_X = 2
local GREEN_Y = 1
local GREEN_DIR = 4	-- rt
local BLUE_X = 1
local BLUE_Y = 2
local BLUE_DIR = 1	-- up


local function entered_red(s)
	return s.x == RED_X and s.y == RED_Y
end

local function entered_green(s)
	return s.x == GREEN_X and s.y == GREEN_Y
end

local function entered_blue(s)
	return s.x == BLUE_X and s.y == BLUE_Y
end

local function entered_park_start(s)
	return s.x == 0 and s.y == -1
end

local function left_up_corner(s)
	return s.x == MIN_X and s.y == MAX_Y
end

local function right_up_corner(s)
	return s.x == MAX_X and s.y == MAX_Y
end

local function make_move(s)
	if s.dir == s.DIR_UP then 
		s.y = s.y + 1
	elseif s.dir == s.DIR_DN then 
		s.y = s.y - 1
	elseif s.dir == s.DIR_RT then 
		s.x = s.x + 1
	elseif s.dir == s.DIR_LT then 
		s.x = s.x - 1
	end
	if (entered_red(s)   and s.dir ~= RED_DIR) or
	   (entered_green(s) and s.dir ~= GREEN_DIR) or
	   (entered_blue(s)  and s.dir ~= BLUE_DIR) then
	   -- телепортируемся
	   s.x = 0
	   s.y = 2
	end
	-- выбирает новое rnd здесь, т.к. select_pic вызывается два раза!
	local r = rnd(3)
	while r == s.rnd_val do
		r = rnd(3)
	end
	s.rnd_val = r
end

local function select_pic(s)
	if left_up_corner(s) then
		return 'gfx/park/fog_ul.jpg'
	elseif right_up_corner(s) then
		return 'gfx/park/fog_ur.jpg'
-- 	Некрасиво, что повторяются одинаковые картинки слева/справа
--	elseif s.x == MIN_X then
--		return 'gfx/park/fog_l.jpg'
--	elseif s.x == MAX_X then
--		return 'gfx/park/fog_r.jpg'
	else
		return 'gfx/park/fog_'..s.rnd_val..'.jpg'
	end
end

-- ЛАБИРИНТ
park = room {
	nam = 'Парк';
	-- константы направлений
	DIR_UP = 1;
	DIR_DN = 2;
	DIR_LT = 3;
	DIR_RT = 4;
	-- сохраняемые переменные
	var {
		dots = 0,			-- число шагов
		x = 0, y = 0,		-- текущие координаты
		dir = 0,			-- направление перехода
		rnd_val = 1			-- rnd для выбора картинки
	};
	pic = function(s)
	    return select_pic(s)
	end;
	dsc = function(s)
		local st = 'Я нахожусь где-то в парке, кругом туман.'
		local i
		for i = 1, s.dots do
			st = st..'.'
		end
		if s.dots >= 10 then
			s.dots = 0
			if rnd(2) == 1 then
				st = 'Неужели я заблудился? Проклятый туман!'
			else
				st = 'Чувствую себя ёжиком в тумане.'
			end
		end
--		st = st..' ('..s.x..', '..s.y..')'
		-- формирование возможных переходов
		st = st..'^'
		if s.y < MAX_Y then	st = st..'^{forw|вперёд}' else st = st..'^[ X ]' end
		if s.x > MIN_X then	st = st..'^^{left|влево}' else st = st..'^^[ X ]' end
		st = st..' . . . . . . . . . . '
		if s.x < MAX_X then	st = st..'{rite|вправо}'  else st = st..'[ X ]' end
		if (s.y > MIN_Y) or (s.x == 0 and s.y == 0) then st = st..'^^{back|назад}' else st = st..'^^[ X ]' end
		return txtc(st)
	end;
	obj = {
		xact('forw', code[[park.dir = park.DIR_UP; goto(park);]]),
		xact('left', code[[park.dir = park.DIR_LT; goto(park);]]),
		xact('rite', code[[park.dir = park.DIR_RT; goto(park);]]),
		xact('back', code[[park.dir = park.DIR_DN; goto(park);]]),
	};
	exit = function(s)
		s.dots = s.dots + 1
		make_move(s)
		if entered_red(s) then
			s.x = RED_X
			s.y = RED_Y - 1
			s.dir = 0
			goto(hollow_red)
		elseif entered_green(s) then
			s.x = GREEN_X - 1
			s.y = GREEN_Y
			s.dir = 0
			goto(hollow_green)
		elseif entered_blue(s) then
			s.x = BLUE_X
			s.y = BLUE_Y - 1
			s.dir = 0
			goto(hollow_blue)
		elseif entered_park_start(s) then
			s.x = 0
			s.y = 0
			s.dir = 0
			s.dots = 0
			goto(park_start)
			return 'Неужели я выбрался из этого тумана!?'
		end
	end;
};


-- ДЕРЕВО С ДУПЛОМ (3 штуки)
hollow_red = room {
	nam = 'Дерево с дуплом';
	pic = 'gfx/park/hollow_red.jpg';
	dsc = [[
		Внезапно дорожка закончилась и я вышел к дереву с {hollow|дуплом}.
		^^
		{back|Назад}
	]];
	obj = {
		xact('hollow', code[[goto(child_cache_red);]]),
		xact('back',   code[[goto(park);]])
	};
};

hollow_green = room {
	nam = 'Дерево с дуплом';
	pic = 'gfx/park/hollow_green.jpg';
	dsc = [[
		Внезапно дорожка закончилась и я вышел к дереву с {hollow|дуплом}.
		^^
		{back|Назад}
	]];
	obj = {
		xact('hollow', code[[goto(child_cache_green);]]),
		xact('back',   code[[goto(park);]])
	};
};

hollow_blue = room {
	nam = 'Дерево с дуплом';
	pic = 'gfx/park/hollow_blue.jpg';
	dsc = [[
		Внезапно дорожка закончилась и я вышел к дереву с {hollow|дуплом}.
		^^
		{back|Назад}
	]];
	obj = {
		xact('hollow', code[[goto(child_cache_blue);]]),
		xact('back',   code[[goto(park);]])
	};
};

-- ДЕТСКИЕ ТАЙНИКИ (3 штуки, copy-paste)
child_cache_red = room {
	nam = 'Детский тайник';
	pic = function(s)
		if not cc_lock_red.open then 
			p 'gfx/child_cache/red.jpg'
		else
			if taken('red_dart') then
				p 'gfx/child_cache/red_dart_taken.jpg'
			else
				p 'gfx/child_cache/red_open.jpg'
			end
		end
	end;
	obj = { 
		'cc_lock_red'
	};
	way = { 'hollow_red' };
};

-- удаляет из инвентаря ключ, если открыли все 3 сундука
function remove_key_teddy()
	if cc_lock_red.open and cc_lock_green.open and cc_lock_blue.open then
		remove('key_teddy', me())
	end
end

cc_lock_red = obj {
	nam = 'замок на детском тайнике';
	var {
		open = false
	};
	dsc = function(s)
		if not s.open then
			p 'В дупле я обнаружил детский тайник. Интересно, какие сокровища там спрятаны, но сундучок закрыт на {замок}.'
		else
			p 'В дупле я обнаружил детский тайник. У меня в детстве тоже был такой.'
		end
	end;
	act = function(s)
		if not s.open then
			p 'Закрыто. Надо найти ключ от детского сундука.'
		end
	end;
	used = function(s, w)
		if w == key_teddy and not s.open then
			s.open = true
			remove_key_teddy()
			objs():add('red_dart')
			return 'Ключик из плюшевого медвежонка подошёл.'
		end
	end;
};

child_cache_green = room {
	nam = 'Детский тайник';
	pic = function(s)
		if not cc_lock_green.open then 
			p 'gfx/child_cache/green.jpg'
		else
			if taken('green_dart') then
				p 'gfx/child_cache/green_dart_taken.jpg'
			else
				p 'gfx/child_cache/green_open.jpg'
			end
		end
	end;
	obj = { 
		'cc_lock_green'
	};
	way = { 'hollow_green' };
};

cc_lock_green = obj {
	nam = 'замок на детском тайнике';
	var {
		open = false
	};
	dsc = function(s)
		if not s.open then
			p 'В дупле я обнаружил детский тайник. Любопытно было бы взглянуть, что там, но сундучок закрыт на {замок}.'
		else
			p 'В дупле я обнаружил детский тайник.'
		end
	end;
	act = function(s)
		if not s.open then
			p 'Мне не хочется ломать детский сундучок, надо найти ключ.'
		end
	end;
	used = function(s, w)
		if w == key_teddy and not s.open then
			s.open = true
			remove_key_teddy()
			objs():add('green_dart')
			return 'Маленький ключик открыл замок.'
		end
	end;
};

child_cache_blue = room {
	nam = 'Детский тайник';
	pic = function(s)
		if not cc_lock_blue.open then 
			p 'gfx/child_cache/blue.jpg'
		else
			if taken('blue_dart') then
				p 'gfx/child_cache/blue_dart_taken.jpg'
			else
				p 'gfx/child_cache/blue_open.jpg'
			end
		end
	end;
	obj = { 
		'cc_lock_blue'
	};
	way = { 'hollow_blue' };
};

cc_lock_blue = obj {
	nam = 'замок на детском тайнике';
	var {
		open = false
	};
	dsc = function(s)
		if not s.open then
			p 'В дупле я обнаружил детский тайник. Мне не терпится узнать, что же спрятано там, но сундучок закрыт на {замок}.'
		else
			p 'В дупле я обнаружил детский тайник.'
		end
	end;
	act = function(s)
		if not s.open then
			p 'Закрыто. Надо найти ключ от детского сундука.'
		end
	end;
	used = function(s, w)
		if w == key_teddy and not s.open then
			s.open = true
			remove_key_teddy()
			objs():add('blue_dart')
			return 'Ключ из плюшевого медвежонка подошёл.'
		end
	end;
};
