-- МИНИ-ИГРА ЗОДИАК (типа игры в 15, но свободны 2 клетки)
-- Вход и выход: комната house_hall_wall
-- При успешном решении устанавливает zodiak_puzzle.solved = true
-- Все объекты и функции, кроме init_zd_board(), объявлены local

-- количество клеток доски по горизонтали и вертикали
local BOARD_X = 3
local BOARD_Y = 5
-- положение левого верхнего угла доски в пикселах
local LEFT_X = 94
local TOP_Y  = 30
-- размер клетки доски в пикселах
local DELTA_X = 84
local DELTA_Y = 86

-- двумерный массив zd_board[ BOARD_Y ][ BOARD_X ], индексы с 1
-- содержит индексы фишек (1..12), 0 - пустая, 13 - луна (неподвижная)
-- сохраняется в save-файле
global {
	zd_board = {};
}

function init_zd_board()
	local y
	zd_board = {}
	for y = 1, BOARD_Y do
		zd_board[y] = {}
	end
	zd_board[1] = {8,  10, 3}
	zd_board[2] = {1,  6,  7}
	zd_board[3] = {4,  12, 2}
	zd_board[4] = {0,  9,  5}
	zd_board[5] = {13, 0, 11}
end

local pic_name = {
	[0] = 'gfx/zodiak_puzzle/empty.jpg',

	[1] = 'gfx/zodiak_puzzle/fire_ram.jpg',
	[2] = 'gfx/zodiak_puzzle/fire_lion.jpg',
	[3] = 'gfx/zodiak_puzzle/fire_archer.jpg',

	[4] = 'gfx/zodiak_puzzle/earth_bull.jpg',
	[5] = 'gfx/zodiak_puzzle/earth_maiden.jpg',
	[6] = 'gfx/zodiak_puzzle/earth_seagoat.jpg',

	[7] = 'gfx/zodiak_puzzle/air_twins.jpg',
	[8] = 'gfx/zodiak_puzzle/air_scales.jpg',
	[9] = 'gfx/zodiak_puzzle/air_water_bearer.jpg',

	[10] = 'gfx/zodiak_puzzle/water_crab.jpg',
	[11] = 'gfx/zodiak_puzzle/water_scorpion.jpg',
	[12] = 'gfx/zodiak_puzzle/water_fish.jpg',

	[13] = 'gfx/zodiak_puzzle/moon.jpg',
}

local function print_board()
	local y
	for y = 1, BOARD_Y do
		print(zd_board[y][1], zd_board[y][2], zd_board[y][3])
	end
end

local function cell_to_pixel(ix, iy)
	local x = LEFT_X + (ix - 1)*DELTA_X
	local y = TOP_Y  + (iy - 1)*DELTA_Y
	return x, y
end

local function draw_board(s)
	local x, y, ix, iy, fname
	local pict = 'gfx/zodiak_puzzle/_board.jpg'
	for iy = 1, BOARD_Y do
		for ix = 1, BOARD_X do
			x, y = cell_to_pixel(ix, iy)
			fname = pic_name[ zd_board[iy][ix] ]
			pict = pict..';'..fname..'@'..x..','..y
		end
	end		 
	if s.selected then
		x, y = cell_to_pixel(s.sel_ix, s.sel_iy)
		pict = pict..';gfx/zodiak_puzzle/frame.png@'..x..','..y
	end
	if s.solved then
		pict = pict..';gfx/zodiak_puzzle/solved.png@55,310'
	end
	return pict
end

-- конструктор объекта фишка
local function new_piece(ix_, iy_)
	local p = { 
		ix = 0;
		iy = 0;
	}
	p.ix = ix_
	p.iy = iy_
	return p
end

-- если (x,y) не принадлежат полю, возвращает new_piece(0,0)
local function select_piece(x, y)
	local ix = 0
	local iy = 0
	if (x >= LEFT_X) and (x < LEFT_X + BOARD_X * DELTA_X) and 
	   (y >= TOP_Y)  and (y < TOP_Y  + BOARD_Y * DELTA_Y) then
		ix = (x - LEFT_X) / DELTA_X
		iy = (y - TOP_Y) / DELTA_Y
		ix = math.floor(ix) + 1
		iy = math.floor(iy) + 1
	end
--	print('click pos: '..x..', '..y..'   selected: '..ix..', '..iy)
	return new_piece(ix, iy)
end

-- находится в поле и содержит фишку 1..12
local function valid_piece(p)
	return p.ix ~= 0 and p.iy ~= 0 and 
		1 <= zd_board[p.iy][p.ix] and zd_board[p.iy][p.ix] <= 12
end

-- находится в поле и содержит фишку 0
local function is_empty(p)
	return p.ix ~= 0 and p.iy ~= 0 and zd_board[p.iy][p.ix] == 0
end

local function can_move_left(p)
	return p.ix > 1 and zd_board[p.iy][p.ix - 1] == 0
end

local function can_move_right(p)
	return p.ix < BOARD_X and zd_board[p.iy][p.ix + 1] == 0 
end

local function can_move_up(p)
	return p.iy > 1 and zd_board[p.iy - 1][p.ix] == 0 
end

local function can_move_down(p)
	return p.iy < BOARD_Y and zd_board[p.iy + 1][p.ix] == 0
end

-- сколько рядом по горизонтали или вертикали свободных клеток
local function empty_neibours(p)
	local cnt = 0
	if can_move_left(p) then
		cnt = cnt + 1
	end
	if can_move_right(p) then
		cnt = cnt + 1
	end
	if can_move_up(p) then
		cnt = cnt + 1
	end
	if can_move_down(p) then
		cnt = cnt + 1
	end
	return cnt
end

-- получить свободную клетку-соседа для p (гарантированно есть!) 
local function get_empty_neibour(p)
	if can_move_left(p) then
		return new_piece(p.ix - 1, p.iy)
	end
	if can_move_right(p) then
		return new_piece(p.ix + 1, p.iy)
	end
	if can_move_up(p) then
		return new_piece(p.ix, p.iy - 1)
	end
	if can_move_down(p) then
		return new_piece(p.ix, p.iy + 1)
	end
	error('Empty neibour not found; should be one!')
end

local function equal(p1, p2)
	return p1.ix == p2.ix and p1.iy == p2.iy
end

-- имеется ли путь из p1 в p2 по пустым клеткам? p2 - пустая
local function is_path(p1, p2)
	local cnt_empty = empty_neibours(p1)
	if cnt_empty == 2 then
		return true
	elseif cnt_empty == 1 then
		local p = get_empty_neibour(p1)
		return equal(p, p2) or empty_neibours(p) > 0
	else
		error('Expected 1 or 2, got ', cnt_empty)
	end
end

local function swap_pieces(p1, p2)
	local tmp = zd_board[p1.iy][p1.ix]
	zd_board[p1.iy][p1.ix] = zd_board[p2.iy][p2.ix]
	zd_board[p2.iy][p2.ix] = tmp
end

local function check_for_sol()
	local sum1 = zd_board[1][1] + zd_board[1][2] + zd_board[1][3]
	local sum2 = zd_board[2][1] + zd_board[2][2] + zd_board[2][3]
	local sum3 = zd_board[3][1] + zd_board[3][2] + zd_board[3][3]
	local sum4 = zd_board[4][1] + zd_board[4][2] + zd_board[4][3]
--	print(sum1, sum2, sum3, sum4)
	return sum1 == 6 and sum2 == 15 and sum3 == 24 and sum4 == 33
end


-- КОМНАТА
zodiak_puzzle = room {
	nam = 'Мини-игра Зодиак';
	hideinv = true;
	var {
		solved = false,        	-- головоломка решена?
		selected = false,		-- выбрана ли фишка?
		sel_ix = 0, sel_iy = 0	-- координаты выбранной фишки
	};
	enter = function(s)
		-- всегда сбрасываем выделение поля
		s.selected = false
	end;
	pic = function(s)
		return draw_board(s)
	end;
	click = function(s, x, y)
		-- требуется ещё 1 клик, чтобы выйти из мини-игры после решения!
		if s.solved then
			goto('house_hall_wall')
		end
		-- обычная обработка
		local p = select_piece(x, y)
		if not s.selected then
			if valid_piece(p) and empty_neibours(p) > 0 then
				s.sel_ix, s.sel_iy = p.ix, p.iy
				s.selected = true
				return true  -- перерисовывает сцену!
			end
		else
			local p1 = new_piece(s.sel_ix, s.sel_iy)
			if is_empty(p) and is_path(p1, p) then
				swap_pieces(p1, p)
				s.selected = false
				if check_for_sol() then
					s.solved = true
					print('Zodiak puzzle solved!')
				end
				return true  -- перерисовывает сцену!
			end
		end
	end;
	obj = { 
		vway('exit', 
			'Используя свободные клетки, расставьте зодиаки по стихиям.^{Выйти}', 
			'house_hall_wall')
	};
};
