-- МИНИ-ИГРА КНИГИ (обмен 2-х книг местами)
-- вход: комната cabinet_bookcase
-- выход: комната cabinet
-- при успешном решении устанавливает book_puzzle.solved = true

-- количество томов
local CNT_BOOKS = 8
-- положение левого верхнего угла 1-го тома в пикселах
local LEFT_X = 41
local TOP_Y  = 96
-- размер тома в пикселах
local BOOK_X = 52
local BOOK_Y = 228

-- массив bk_board[ CNT_BOOKS ], индексы с 1
-- содержит индексы томов (1..8), 0 - если пустая
-- сохраняется в save-файле
global {
	bk_board = {3, 4, 7, 2, 5, 1, 8, 6}
}

local pic_name = {
	[0] = 'gfx/book_puzzle/empty.jpg',
	[1] = 'gfx/book_puzzle/1889.jpg',
	[2] = 'gfx/book_puzzle/1891.jpg',
	[3] = 'gfx/book_puzzle/1893.jpg',
	[4] = 'gfx/book_puzzle/1895.jpg',
	[5] = 'gfx/book_puzzle/1898.jpg',
	[6] = 'gfx/book_puzzle/1900.jpg',
	[7] = 'gfx/book_puzzle/1901.jpg',
	[8] = 'gfx/book_puzzle/1904.jpg',
}

local function cell_to_pixel(i)
	local x = LEFT_X + (i - 1)*BOOK_X
	local y = TOP_Y
	return x, y
end

local function draw_board(s)
	local x, y, i, fname
	local pict = 'gfx/book_puzzle/_board.jpg'
	for i = 1, CNT_BOOKS do
		x, y = cell_to_pixel(i)
		fname = pic_name[ bk_board[i] ]
		pict = pict..';'..fname..'@'..x..','..y
	end		 
	if s.sel_x ~= 0 then
		x, y = cell_to_pixel(s.sel_x)
		pict = pict..';gfx/book_puzzle/frame.png@'..x..','..y
	end
	if s.solved then
		pict = pict..';gfx/book_puzzle/solved.png@100,210'
	end
	return pict
end

-- если (x,y) не принадлежат полю, возвращает 0, иначе индекс тома (1..8)
local function select_piece(x, y)
	local ix = 0
	if (x >= LEFT_X) and (x < LEFT_X + CNT_BOOKS * BOOK_X) and 
	   (y >= TOP_Y)  and (y < TOP_Y  + BOOK_Y) then
		ix = (x - LEFT_X) / BOOK_X
		ix = math.floor(ix) + 1
	end
--	print('click pos: '..x..', '..y..'   selected: '..ix)
	return ix
end

local function swap_pieces(i, j)
	local tmp = bk_board[i]
	bk_board[i] = bk_board[j]
	bk_board[j] = tmp
end

local function check_for_sol()
	local i
	for i = 1, CNT_BOOKS do
    	if bk_board[i] ~= i then
    		return false
		end
	end
	return true
end


-- КОМНАТА
book_puzzle = room {
	nam = 'Мини-игра Книги';
	hideinv = true;
	var {
		solved = false,  -- головоломка решена?
		sel_x = 0		 -- индекс выбранного тома (1..8, 0=не выбран)
	};
	enter = function(s)
		-- всегда снимает предыдущее выделение поля
		s.sel_x = 0
	end;
	pic = function(s)
		return draw_board(s)
	end;
	click = function(s, x, y)
		-- требуется ещё 1 клик, чтобы выйти из мини-игры после решения!
		if s.solved then
			goto('cabinet')
		end
		local ix = select_piece(x, y)
		if ix ~= 0 then
			if s.sel_x == 0 then
				s.sel_x = ix
			elseif ix ~= s.sel_x then
				swap_pieces(ix, s.sel_x)
				s.sel_x = 0
				if check_for_sol() then
					s.solved = true
					print('Book puzzle solved!')
				end
			end
			return true  -- перерисовывает сцену!
		end
	end;
	obj = { 
		vway('exit', 
			'Меняя тома местами, расставьте книги в правильном порядке.^^{Выйти}', 
			'cabinet_bookcase')
	};
};
