-- МИНИ-ИГРА ШКАТУЛКА (поворот 5 сегментов)
-- вход и выход: комната casket
-- при успешном решении устанавливает casket_puzzle.solved = true 

-- количество сегментов
local CNT_SEG = 5
-- индексы сегментов
local SEG_A = 1
local SEG_B = 2
local SEG_C = 3
local SEG_D = 4
local SEG_E = 5
-- координаты левого-верхнего угла сегментов
local LEFT_X = {87, 262, 263,  87, 187}
local TOP_Y  = {77,  77, 247, 247, 173}
local SEG_NAME = {'a', 'b', 'c', 'd', 'e'}
-- размеры сегментов-квадратов
local BIG_SIZE = 94		-- (A, B, C, D)
local SMALL_SIZE = 71	-- (E)

-- массив ca_board[CNT_SEG], индексы с 1
-- содержит повороты каждого сегмента (1..4)
-- сохраняется в save-файле
global {
	ca_board = {1, 1, 1, 1, 1}
}

local function draw_board(s)
	local i
	local pict = 'gfx/casket_puzzle/_board.jpg'
	for i = 1, CNT_SEG do
		pict = pict..';gfx/casket_puzzle/'..SEG_NAME[i]..ca_board[i]..
						'.jpg@'..LEFT_X[i]..','..TOP_Y[i]
	end
	if s.solved then
		pict = pict..';gfx/casket_puzzle/solved.png@66,200'
	end
	return pict
end

-- если (x,y) не принадлежат полю, возвращает 0, иначе индекс сегмента (1..5)
local function select_piece(x, y)
	local i 
	for i = 1, CNT_SEG do
		local seg_size = BIG_SIZE
		if i == SEG_E then
			seg_size = SMALL_SIZE
		end
		if (x >= LEFT_X[i]) and (x < LEFT_X[i] + seg_size) and 
	   	   (y >= TOP_Y[i])  and (y < TOP_Y[i]  + seg_size) then
			return i
		end
	end
	return 0
end

-- повернуть i-ый сегмент на 90 градусов вправо
local function turn_right(i)
	ca_board[i] = ca_board[i] + 1
	if ca_board[i] == 5 then
		ca_board[i] = 1
	end
end

-- нажать на i-ый сегмент
local function press(i)
	if i == SEG_A then
		turn_right(SEG_A)
	elseif i == SEG_B then
		turn_right(SEG_B)
		turn_right(SEG_C)
		turn_right(SEG_D)
	elseif i == SEG_C then
		turn_right(SEG_A)
		turn_right(SEG_C)
	elseif i == SEG_D then
		turn_right(SEG_A)
		turn_right(SEG_C)
		turn_right(SEG_D)
	elseif i == SEG_E then
		turn_right(SEG_A)
		turn_right(SEG_B)
		turn_right(SEG_C)
		turn_right(SEG_D)
		turn_right(SEG_E)
	end
end

local function check_for_sol()
	return ca_board[SEG_A] == 4 and ca_board[SEG_B] == 3 and 
	       ca_board[SEG_C] == 1 and ca_board[SEG_D] == 2 and 
	       ca_board[SEG_E] == 4 
end


-- КОМНАТА
casket_puzzle = room {
	nam = 'Мини-игра Шкатулка';
	hideinv = true;
	var {
		solved = false
	};
	pic = function(s)
		return draw_board(s)
	end;
	click = function(s, x, y)
		-- требуется ещё 1 клик, чтобы выйти из мини-игры после решения!
		if s.solved then
			goto('casket')
		end
		-- обычная обработка
		local p = select_piece(x, y)
		if p ~= 0 then
			press(p)
			if check_for_sol() then
				s.solved = true
				print('Casket puzzle solved!')
			end
			return true  -- перерисовывает сцену!
		end
	end;
	obj = { 
		vway('exit', 
			'Вращая сегменты замка, соберите единый узор.^{Выйти}', 
			'casket')
	};
};
