-- МИНИ-ИГРА АЛТАРЬ (замок, который поворачивает/переносит 4 фишки)
-- вход и выход: комната church_altar
-- при успешном решении устанавливает altar_puzzle.solved = true 

-- количество фишек и обычных клеток под фишки
local CNT_SEG = 4
local CNT_CELLS = 7
-- индексы клетки на диски, они же направления поворота диска
local DISC_UP = 8
local DISC_RT = 9
-- индекс клетки, если кликнули по диску
local SEG_DISC = 10
local SEG_NAME = {'d', 'e', 'u', 's'}
--[[
		[1] [2] [3] [4]
		    [8]     [5]     [8,9] - это 2 положения фишки в зависимости от 
	    	    [9] [6]   	        поворота диска (вверх, вправо)
	        	    [7]
]]
-- массив al_board[2][CNT_SEG], индексы с 1
-- сохраняется в save-файле
global {
	al_board = {
		{4, 3, 2, 1}, 	-- номер клетки (1..9), где лежит i-ая фишка 
		{1, 2, 4, 3}	-- и повороты каждой фишки (1..4)
	};
	al_disc = 8			-- поворот диска (8..9)
}

-- координаты левого-верхнего угла сегментов (9 штук)
local LEFT_X = {80,  143, 207, 270, 270, 270, 270, 143, 193}
local TOP_Y  = {124, 124, 124, 124, 188, 258, 326, 206, 258}
-- координаты диска
local DISC_X = 82
local DISC_Y = 196
-- размеры фишки и диска
local CELL_SIZE = 56	
local DISC_SIZE = 175

local function draw_disc()
	local pict
	if al_disc == DISC_UP then
		pict = ';gfx/altar_puzzle/disc_up.jpg@'..DISC_X..','..DISC_Y
	else
		pict = ';gfx/altar_puzzle/disc_rt.jpg@'..DISC_X..','..DISC_Y
	end
	return pict..';gfx/altar_puzzle/frame2.png@'..LEFT_X[al_disc]..','..TOP_Y[al_disc]
end

local function draw_board(s)
	local pict = 'gfx/altar_puzzle/_board.jpg'..draw_disc()
	local i
	for i = 1, CNT_CELLS do
		pict = pict..';gfx/altar_puzzle/frame2.png@'..LEFT_X[i]..','..TOP_Y[i]
	end
	for i = 1, CNT_SEG do
		local pos = al_board[1][i]
		local rot = al_board[2][i]
		pict = pict..';gfx/altar_puzzle/chip56.jpg@'..LEFT_X[pos]..','..TOP_Y[pos]
		pict = pict..';gfx/altar_puzzle/'..SEG_NAME[i]..rot..
						'.png@'..(LEFT_X[pos]+5)..','..(TOP_Y[pos]+5)
	end
	if s.sel ~= 0 and s.sel ~= SEG_DISC then
		pict = pict..';gfx/altar_puzzle/frame.png@'..LEFT_X[s.sel]..','..TOP_Y[s.sel]
	end
	if s.solved then
		pict = pict..';gfx/altar_puzzle/solved.png@66,200'
	end
	return pict
end

-- если (x,y) не принадлежат полю, возвращает 0, иначе индекс клетки
local function select_piece(x, y)
	local i 
	for i = 1, CNT_CELLS + 2 do
		if (x >= LEFT_X[i]) and (x < LEFT_X[i] + CELL_SIZE) and 
	   	   (y >= TOP_Y[i])  and (y < TOP_Y[i]  + CELL_SIZE) then
	   		if (i <= CNT_CELLS) or (i == al_disc) then
				return i
			end
		end
	end
	if (x >= DISC_X) and (x < DISC_X + DISC_SIZE) and 
   	   (y >= DISC_Y) and (y < DISC_Y + DISC_SIZE) then
		return SEG_DISC
	end
	return 0
end

-- получить индекс фишки в клетке p или 0, если её там нет
local function get_chip(p)
	local i
	for i = 1, CNT_SEG do
		if al_board[1][i] == p then
			return i
		end
	end
	return 0
end

-- повернуть i-ый сегмент на 90 градусов влево
local function turn_left(i)
	al_board[2][i] = al_board[2][i] - 1
	if al_board[2][i] == 0 then
		al_board[2][i] = 4
	end
end

-- повернуть i-ый сегмент на 90 градусов вправо
local function turn_right(i)
	al_board[2][i] = al_board[2][i] + 1
	if al_board[2][i] == 5 then
		al_board[2][i] = 1
	end
end

-- повернуть диск, возможно, вместе с фишкой
local function rotate_disc(s)
	local p = get_chip(al_disc)
	if al_disc == DISC_UP then
		if s.sel == al_disc then 
			s.sel = DISC_RT
		end
		al_disc = DISC_RT
		if p ~= 0 then
			al_board[1][p] = DISC_RT
			turn_right(p)
		end
	else
		if s.sel == al_disc then 
			s.sel = DISC_UP
		end
		al_disc = DISC_UP
		if p ~= 0 then
			al_board[1][p] = DISC_UP
			turn_left(p)
		end
	end
end

-- имеется ли фишка в клетке p?
local function is_chip(p)
	return get_chip(p) ~= 0
end

-- свободна ли клетка p?
local function is_free(p)
	return get_chip(p) == 0
end

-- можно ли передвинуть фишку с клетки from в клетку to?
-- клетка to гарантированно свободна
local function can_move(from, to)
	-- если целевая выбрана на диске, берем для проверки смежную 
	if to == DISC_UP then
		to = 2
	elseif to == DISC_RT then
		to = 6 
	end
	if from == DISC_UP then
		if is_free(2) then from = 2 else return false end
	elseif from == DISC_RT then
		if is_free(6) then from = 6 else return false end
	end
	local p
	if to < from then
		-- все клетки от to до from-1 должны быть свободны
		for p = to, from - 1 do
			if not is_free(p) then
				return false
			end
		end
		return true
	elseif to > from then
		-- все клетки от from+1 до to должны быть свободны
		for p = from + 1, to do
			if not is_free(p) then
				return false
			end
		end
		return true
	else
		return true
	end
end

-- передвинуть фишку с клетки from в клетку to
local function make_move(from, to)
	local ch = get_chip(from)
	al_board[1][ch] = to
end

local function check_for_sol()
	local sol = {
		{1, 2, 3, 4}, 
		{1, 1, 1, 1}
	};
	local i, j
	for j = 1, 2 do
		for i = 1, CNT_SEG do
    		if al_board[j][i] ~= sol[j][i] then
    			return false
			end
		end
	end
	return true
end

-- КОМНАТА
altar_puzzle = room {
	nam = 'Мини-игра Алтарь';
	hideinv = true;
	var {
		solved = false,
		sel = 0			 -- индекс выбранной клетки (1..9, 0=не выбран)
	};
	enter = function(s)
		s.sel = 0
	end;
	pic = function(s)
		return draw_board(s)
	end;
	click = function(s, x, y)
		-- требуется ещё 1 клик, чтобы выйти из мини-игры после решения!
		if s.solved then
			goto('church_altar')
		end
		-- обычная обработка
		local p = select_piece(x, y)
		if p ~= 0 then
			if p == SEG_DISC then
				rotate_disc(s)
			else
				if s.sel == 0 then
					-- если фишка была не выбрана - помечаем
					if is_chip(p) then
						s.sel = p
					end
				else
					if s.sel ~= p then
						-- если кликнули на другую фишку - выбираем её
						if is_chip(p) then
							s.sel = p
						-- если можем переместить - двигаем
						elseif can_move(s.sel, p) then
							make_move(s.sel, p)
							s.sel = 0
						end
					end
				end
			end
			if check_for_sol() then
				s.solved = true
				print('Altar puzzle solved!')
			end
			return true  -- перерисовывает сцену!
		end
	end;
	obj = { 
		vway('exit', 
			'Поворачивая фишки с помощью диска, соберите ключевое слово.^{Выйти}', 
			'church_altar')
	};
};
